{% extends "base.html" %}

{% block site_meta %}
    {{ super() }}
    <meta name="keywords" content="prompt engineering, prompt management, LLM, AI">
    <meta name="robots" content="index, follow">
    <meta name="language" content="English">
{% endblock %}
