/* 
 * Original credit for this admonition implementation goes to the team at Pydantic
 * https://github.com/pydantic/pydantic/blob/main/docs/extra/tweaks.css
*/

/* API documentation link admonition */
:root {
  --md-primary-fg-color:        #6366f1;
  --md-primary-fg-color--light: #6366f1;
  --md-primary-fg-color--dark:  #6366f1;
}

:root {
    --md-admonition-icon--api: url('data:image/svg+xml;charset=utf-8,<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d="M7 7H5a2 2 0 0 0-2 2v8h2v-4h2v4h2V9a2 2 0 0 0-2-2m0 4H5V9h2m7-2h-4v10h2v-4h2a2 2 0 0 0 2-2V9a2 2 0 0 0-2-2m0 4h-2V9h2m6 0v6h1v2h-4v-2h1V9h-1V7h4v2Z"/></svg>')
  }
  .md-typeset .admonition.api, .md-typeset details.api {
    border-color: #448aff;
  }
  .md-typeset .api > .admonition-title, .md-typeset .api > summary {
    background-color: #448aff1a;
  }
  .md-typeset .api > .admonition-title::before, .md-typeset .api > summary::before {
    background-color: #448aff;
    -webkit-mask-image: var(--md-admonition-icon--api);
            mask-image: var(--md-admonition-icon--api);
  }
